import sqlite3
from pprint import pprint
from datetime import date, timedelta
from sklearn.neighbors import KNeighborsClassifier

#------------------------------------------------
#------------------------------------------------
def str2date(datestr):
    (y,m,d) = datestr.split('-')
    return date(int(y), int(m), int(d))

#------------------------------------------------
#------------------------------------------------
def getFeatures(db, sym, xdata, nfeatures, windays):
    c = db.cursor()
    flist = []
    for row in xdata:
        yesterday = row - timedelta(days=1)
        lowdate = row - timedelta(days=(windays+1))
        t = \
        (sym,lowdate.isoformat(),yesterday.isoformat(),nfeatures)
        c.execute('SELECT open FROM stock WHERE sym=? AND sdate >= ? \
                AND sdate <= ? ORDER BY sdate DESC LIMIT ?', t)
        tlist = c.fetchall()
        f = [ t[0] for t in tlist ]
        flist.append(f) 

    return flist

def getMeanAbsErr(predicted, actual):
    n = len(predicted)
    totalerr = 0
    for i in range(n):
        totalerr = totalerr + abs(predicted[i]-actual[i])
    return (totalerr/n)

def getKNNClassifier(db, sym, start, end, kneighbors, nfeatures, windays):
    t = (sym,start,end)
    c.execute('SELECT sdate, open FROM stock WHERE sym=? AND sdate >= ? \
                AND sdate <= ? ORDER BY sdate', t)
    stkdata = c.fetchall()

    xlist = []
    ylist = []
    for row in stkdata:
        sdate = str2date(row[0])
        xlist.append(sdate)
        ylist.append(int(row[1]))

    flist = getFeatures(db, sym, xlist, nfeatures, windays)
    myknn = \
        KNeighborsClassifier(n_neighbors=kneighbors,\
            algorithm='kd_tree', p=2, weights='distance')

    myknn.fit(flist, ylist)
    return myknn

conn = sqlite3.connect('stock.db')

c = conn.cursor()

sym = 'XOM'
kneighbors = 3
nfeatures = 5
windays = 12

# date range of training data
trgstart = date(2015,8,1)
trgend   = date(2016,8,1)

# date range of testing data
start = date(2016,8,2)
end   = date(2017,8,1)

# get testing data
t = (sym,start,end)
c.execute('SELECT sdate, open FROM stock WHERE sym=? AND sdate >= ? \
                AND sdate <= ? ORDER BY sdate', t)

stkdata = c.fetchall()


xlist = []
ylist = []
for row in stkdata:
    sdate = str2date(row[0])
    xlist.append(sdate)
    ylist.append(int(row[1]))

tstx = getFeatures(conn, sym, xlist, nfeatures, windays)
tsty = ylist

#pprint(tstx)
#pprint(ylist)

# construct knn classifier using training data date range
myknn = getKNNClassifier(conn, sym, trgstart, trgend, \
                kneighbors, nfeatures, windays)

tsty_predicted = myknn.predict(tstx)
               
pprint(tsty_predicted)

pprint( myknn.score(tstx, tsty) )

pprint( getMeanAbsErr(tsty_predicted, tsty) )


conn.close()
